﻿using Microsoft.Extensions.Logging;
using System;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject.Handlers
{
    /// <summary>
    /// 데이터 요청 핸들러
    /// </summary>
    public class RequestDataHandler : DelegatingHandler
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 로그 기록기
        /// </summary>
        private readonly ILogger<RequestDataHandler> logger;

        /// <summary>
        /// 요청 소스 헤더명
        /// </summary>
        private const string RequestSourceHeaderName = "Request-Source";

        /// <summary>
        /// 요청 소스
        /// </summary>
        private const string RequestSource = "TestProject";

        /// <summary>
        /// 요청 ID 헤더명
        /// </summary>
        private const string RequestIDHeaderName = "Request-Identifier";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RequestDataHandler(logger)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="logger">로그 기록기</param>
        public RequestDataHandler(ILogger<RequestDataHandler> logger)
        {
            this.logger = logger;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 송신하기 (비동기) - SendAsync(httpRequestMessage, cancellationToken)

        /// <summary>
        /// 송신하기 (비동기)
        /// </summary>
        /// <param name="httpRequestMessage">HTTP 요청 메시지</param>
        /// <param name="cancellationToken">취소 토큰</param>
        /// <returns>HTTP 응답 메시지 태스크</returns>
        protected override Task<HttpResponseMessage> SendAsync
        (
            HttpRequestMessage httpRequestMessage,
            CancellationToken  cancellationToken
        )
        {
            Guid identifierGUID = Guid.NewGuid();

            this.logger.LogInformation("Starting request {Identifier}", identifierGUID);

            httpRequestMessage.Headers.Add(RequestSourceHeaderName, RequestSource            );
            httpRequestMessage.Headers.Add(RequestIDHeaderName    , identifierGUID.ToString());

            return base.SendAsync(httpRequestMessage, cancellationToken);
        }

        #endregion
    }
}