﻿using System;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject.Handlers
{
    /// <summary>
    /// 보안 요청 핸들러
    /// </summary>
    public class SecureRequestHandler : DelegatingHandler
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 송신하기 (비동기) - SendAsync(request, cancellationToken)

        /// <summary>
        /// 송신하기 (비동기)
        /// </summary>
        /// <param name="httpRequestMessage">HTTP 요청 메시지</param>
        /// <param name="cancellationToken">취소 토큰</param>
        /// <returns>HTTP 응답 메시지 태스크</returns>
        protected override Task<HttpResponseMessage> SendAsync
        (
            HttpRequestMessage httpRequestMessage,
            CancellationToken  cancellationToken
        )
        {
            if(httpRequestMessage.RequestUri.Scheme == Uri.UriSchemeHttp)
            {
                UriBuilder uriBuilder = new UriBuilder(httpRequestMessage.RequestUri)
                {
                    Scheme = Uri.UriSchemeHttps
                };

                httpRequestMessage.RequestUri = uriBuilder.Uri;
            }

            return base.SendAsync(httpRequestMessage, cancellationToken);
        }

        #endregion
    }
}